﻿

namespace HoistingCrane;

public class DrawingHoistingCrane
{
    /// Класс-сущность
    public EntityHoistingCrane? EntityHoistingCrane { get; private set; }
    /// Ширина окна
    private int? _pictureWidth;
    /// Высота окна
    private int? _pictureHeight;
    /// Левая координата прорисовки автомобиля
    private int? _startPosX;
    /// Верхняя кооридната прорисовки автомобиля
    private int? _startPosY;
    /// Ширина прорисовки автомобиля
    private readonly int _drawningCraneWidth = 110;
    /// Высота прорисовки автомобиля
    private readonly int _drawningCraneHeight = 56;


    /// Инициализация свойств
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="crane">Признак наличия крана</param>
    /// <param name="counterweight">Признак наличия противовеса</param>
    public void Init(int speed, double weight, Color bodyColor, Color
    additionalColor, bool crane, bool counterweight)
    {
        EntityHoistingCrane = new EntityHoistingCrane();
        EntityHoistingCrane.Init(speed, weight, bodyColor, additionalColor, crane, counterweight);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// Установка границ поля
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (width < _drawningCraneWidth || height < _drawningCraneHeight)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;

        if (_startPosX != null || _startPosY != null)
        {
            if (_startPosX + _drawningCraneWidth > _pictureWidth) _startPosX = _pictureWidth - _drawningCraneWidth;
            if (_startPosY + _drawningCraneHeight > _pictureHeight) _startPosY = _pictureHeight - _drawningCraneHeight;
            if (_startPosX < 0) _startPosX = 0;
            if (_startPosY < 0) _startPosY = 0;
        }
        return true;
    }
    /// Установка позиции
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        _startPosX = x;
        _startPosY = y;
        if (_startPosX + _drawningCraneWidth > _pictureWidth) _startPosX = _pictureWidth - _drawningCraneWidth;
        if (_startPosY + _drawningCraneHeight > _pictureHeight) _startPosY = _pictureHeight - _drawningCraneHeight;
        if (_startPosX < 0) _startPosX = 0;
        if (_startPosY < 0) _startPosY = 0;
    }
    /// Изменение направления перемещения
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityHoistingCrane == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityHoistingCrane.Step > 0)
                {
                    _startPosX -= (int)EntityHoistingCrane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityHoistingCrane.Step > 0)
                {
                    _startPosY -= (int)EntityHoistingCrane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityHoistingCrane.Step < _pictureWidth - _drawningCraneWidth)
                {
                    _startPosX += (int)EntityHoistingCrane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityHoistingCrane.Step < _pictureHeight - _drawningCraneHeight)
                {
                    _startPosY += (int)EntityHoistingCrane.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>

    public void DrawTransport(Graphics g)
    {
        if (EntityHoistingCrane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityHoistingCrane.AdditionalColor);
        Brush BodyBrush = new SolidBrush(EntityHoistingCrane.BodyColor);
        Brush CraneBrush = new SolidBrush(Color.Black);


        //корпус
        g.FillRectangle(BodyBrush, _startPosX.Value + 10, _startPosY.Value + 30, 62, 12);
        g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value + 30, 62, 12);
        g.FillRectangle(BodyBrush, _startPosX.Value + 22, _startPosY.Value + 16, 4, 15);
        g.DrawRectangle(pen, _startPosX.Value + 22, _startPosY.Value + 16, 4, 14);
        g.FillRectangle(BodyBrush, _startPosX.Value + 37, _startPosY.Value + 8, 5, 23);
        g.DrawRectangle(pen, _startPosX.Value + 37, _startPosY.Value + 8, 5, 22);

        //стекло
        g.FillRectangle(additionalBrush, _startPosX.Value + 52, _startPosY.Value + 14, 21, 16);
        g.DrawRectangle(pen, _startPosX.Value + 52, _startPosY.Value + 14, 20, 16);

        //гусеницы
        g.DrawLine(pen, _startPosX.Value + 8, _startPosY.Value + 42, _startPosX.Value + 10, _startPosY.Value + 42);
        g.DrawLine(pen, _startPosX.Value + 7, _startPosY.Value + 43, _startPosX.Value + 7, _startPosY.Value + 53);
        g.DrawLine(pen, _startPosX.Value + 8, _startPosY.Value + 54, _startPosX.Value + 12, _startPosY.Value + 54);
        g.DrawLine(pen, _startPosX.Value + 13, _startPosY.Value + 55, _startPosX.Value + 69, _startPosY.Value + 55);
        g.DrawLine(pen, _startPosX.Value + 70, _startPosY.Value + 54, _startPosX.Value + 73, _startPosY.Value + 54);
        g.DrawLine(pen, _startPosX.Value + 74, _startPosY.Value + 53, _startPosX.Value + 74, _startPosY.Value + 43);
        g.DrawLine(pen, _startPosX.Value + 71, _startPosY.Value + 42, _startPosX.Value + 73, _startPosY.Value + 42);

        //колеса
        g.FillEllipse(BodyBrush, _startPosX.Value + 10, _startPosY.Value + 44, 9, 9);
        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 44, 9, 9);
        g.FillEllipse(BodyBrush, _startPosX.Value + 63, _startPosY.Value + 44, 9, 9);
        g.DrawEllipse(pen, _startPosX.Value + 63, _startPosY.Value + 44, 9, 9);

        g.FillEllipse(BodyBrush, _startPosX.Value + 25, _startPosY.Value + 48, 6, 6);
        g.DrawEllipse(pen, _startPosX.Value + 25, _startPosY.Value + 48, 6, 6);
        g.FillEllipse(BodyBrush, _startPosX.Value + 38, _startPosY.Value + 48, 6, 6);
        g.DrawEllipse(pen, _startPosX.Value + 38, _startPosY.Value + 48, 6, 6);
        g.FillEllipse(BodyBrush, _startPosX.Value + 50, _startPosY.Value + 48, 6, 6);
        g.DrawEllipse(pen, _startPosX.Value + 50, _startPosY.Value + 48, 6, 6);

        g.FillEllipse(BodyBrush, _startPosX.Value + 33, _startPosY.Value + 44, 4, 4);
        g.DrawEllipse(pen, _startPosX.Value + 33, _startPosY.Value + 44, 4, 4);
        g.FillEllipse(BodyBrush, _startPosX.Value + 45, _startPosY.Value + 44, 4, 4);
        g.DrawEllipse(pen, _startPosX.Value + 45, _startPosY.Value + 44, 4, 4);

        //кран
        if (EntityHoistingCrane.Crane)
        {
            //балка
            g.FillRectangle(BodyBrush, _startPosX.Value, _startPosY.Value, 110, 9);
            g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value, 109, 8);
            //крюк
            g.FillRectangle(CraneBrush, _startPosX.Value + 107, _startPosY.Value + 8, 3, 13);
            g.FillRectangle(CraneBrush, _startPosX.Value + 102, _startPosY.Value + 18, 8, 3);
            g.FillRectangle(CraneBrush, _startPosX.Value + 102, _startPosY.Value + 16, 3, 5);
        }

        //противовес
        if (EntityHoistingCrane.Counterweight)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value, 10, 9);
            g.DrawRectangle(pen, _startPosX.Value + 35, _startPosY.Value, 9, 8);

            g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 2, _startPosX.Value + 44, _startPosY.Value + 2);
            g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 4, _startPosX.Value + 44, _startPosY.Value + 4);
            g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 6, _startPosX.Value + 44, _startPosY.Value + 6);
        }
    }
}

